local _cx_mini_={}

function cx_mini_fmt_name_accept(sub)
	if sub==' ' then return '\n' end
	if sub~='\n' then return true end
end
function cx_mini_fmt_name(name,accept,width,lines)
	width=width or 8
	lines=lines or 1
	local curline={}
	local result={}
	local cur=0
	for p, c in utf8.codes(name) do
		local sub=utf8.char(c)
		local check=not accept or accept(sub)
		if check==true then
			if c>=255 then cur=cur+2 else cur=cur+1 end
			if cur>width then
				result[#result+1]=table.concat(curline)
				curline={sub}
				if c>=255 then cur=2 else cur=1 end
			else
				curline[#curline+1]=sub
			end
			if cur==width then
				result[#result+1]=table.concat(curline)
				curline={}
				cur=0
			end
			if #result>=lines then return result end
		elseif check=='\n' then
			result[#result+1]=table.concat(curline)
			curline={}
			cur=0
		end
	end
	result[#result+1]=table.concat(curline)
	return result
end

function cx_mini_parseswitch(obj,tbl)
	if type(obj)=='table' then
		local cgy=obj.cgy
		tbl.cgy=cgy
		tbl.cls=obj.cls
		if cgy=='io' then
			local name
			if obj.__name then
				tbl.name=obj.__name
			else
				tbl.name=obj.name and #obj.name>0 and obj.name or obj.vidx
			end
			if obj.__rv then
				tbl.valstr=obj.__rv .. (obj.__unit or '')
			else
				tbl.valstr=''
			end
			if type(obj.type)=='number' then
				tbl.sw=obj.type&1
				tbl.type=obj.type
				tbl.val=obj.val
				if (obj.type&0x80)==0 and tbl.valstr and #tbl.valstr>0 then
					tbl.name=tbl.valstr
				end
			else
				tbl.sw=0
			end
			return
		elseif cgy=='ep' then
			tbl.name=obj.name and #obj.name>0 and obj.name or 'EP'
			tbl.sw=0
			return
		elseif cgy=='ai' then
			tbl.name=obj.name and #obj.name>0 and obj.name or 'AI'
			tbl.sw=obj.stat==3 and 1 or 0
			tbl.stat=obj.stat
			tbl._UI_BT_CNT=obj._UI_BT_CNT
			return
		else
			tbl.name=obj.name and #obj.name>0 and obj.name or '?'
			tbl.sw=0
			return
		end
	end
	tbl.cgy=nil
	return
end

function cx_mini_init()
	local self = setmetatable({mini_objs={}},{__index = _cx_mini_ })
	return self
end


function _cx_mini_:add_touch(path,cmd,idx,flag,x,y,x2,y2,kv)
	local obj={path=path,cmd=cmd,info={},idx,flag,x,y,x2,y2}
	self.mini_objs[idx]=obj;
	if type(kv)=='table' then
		for k,v in pairs(kv) do
			obj[k]=v
		end
	end
	if type(x)=='table' and type(x[1])=='table' then
		for i,rectval in ipairs(x) do
			touchdef(idx,flag&1,rectval[1],rectval[2],rectval[3],rectval[4],i)
		end
	else
		touchdef(idx,flag&1,x,y,x2,y2)
	end
end


function _cx_mini_:get_info(idx,attr,def)
	local v=self.mini_objs[idx]
	v=v and v.info
	if not v then return def end
	if attr then
		local rv=v[attr]
		if vr==nil then return def end
		return rv
	else
		return v
	end
end

function _cx_mini_:doclick(idx,dotrigger,rectidx)
	local v=self.mini_objs[idx]
	if v.doclick then
		v.doclick(v,idx,rectidx)
		return true
	elseif v and v.path then
		local cmd=v.cmd or 'click'
		if type(dotrigger)=='function' then
			cmd=dotrigger(idx,v,cmd) or cmd
		end
		svcreq("epctl",string.format("json:%s:%s",v.path,cmd))
		return true
	end
end

function _cx_mini_:update()
	for k,v in pairs(self.mini_objs) do
		v.obj=svcget(v.path)
		cx_mini_parseswitch(v.obj,v.info)
	end
end

function _cx_mini_:scantouch(dotrigger)
	local mx,my,mb,hotidx,rectidx=touch()
	local trigger
	if not self.drag and hotidx and mb~=0 then
		local focus=self.mini_objs[hotidx]
		if focus and focus.dragcb then
			self.drag=true
			self.drag_idx=hotidx
			self.drag_rectidx=rectidx
		end
	end
	if self.drag then
		local focus=self.mini_objs[self.drag_idx]
		if not focus then
			self.drag=nil
			self.drag_idx=nil
			self.drag_rectidx=nil
		else
			if mb==0 then
				pcall(focus.dragcb,focus,self.drag_idx,mx,my,mb)
				self.drag=nil
				self.drag_idx=nil
				self.drag_rectidx=nil
				return
			end
			pcall(focus.dragcb,focus,self.drag_idx,mx,my,mb)
			return
		end
	end

	if hotidx>0 then
		local obj=self.mini_objs[idx]
		if self.userelease or (obj and obj.userelease) then
			if mb==0 then
				if mb~=self.lmb and self.lhotidx==hotidx then
					if dotrigger then
						self:doclick(hotidx,dotrigger,rectidx)
					end
					trigger=true
				end
			else
				if mb~=self.lmb then
				end
			end
		else
			if mb==1 then
				if mb~=self.lmb then
					if dotrigger then
						self:doclick(hotidx,dotrigger,rectidx)
					end
					trigger=true
				end
			else
				if mb~=self.lmb then
				end
			end
		end
	end
	self.lmx,self.lmy,self.lmb,self.lhotidx=mx,my,mb,hotidx
	return hotidx,trigger,rectidx,mx,my,mb
end

function _cx_mini_:draw(hotidx,trigger)
	for k,v in pairs(self.mini_objs) do
		local info=self:get_info(k)
		if info then
			local color=info.sw==1 and red or blue
			if hotidx==k and trigger then
				color=orange
			end
			rectfill(v[3],v[4],v[5],v[6],color)
			drawTxt_h(info.name,(v[3]+v[5])//2,(v[4]+v[6])//2-16,white,4,32,'default')
		else
			rectfill(v[3],v[4],v[5],v[6],orange)
		end
	end
end

function _cx_mini_:draw2(hotidx,trigger)
	for k,v in pairs(self.mini_objs) do
		local info=self:get_info(k)
		if info then
			local color=info.sw==1 and red or blue
			if hotidx==k and trigger then
				color=orange
			end
			--rectfill(v[3],v[4],v[5],v[6],color)
			drawTxt_h(info.name,(v[3]+v[5])//2,(v[4]+v[6])//2-16,color,4,32,'default')
		else
			--rectfill(v[3],v[4],v[5],v[6],orange)
		end
	end
end


